import java.util.ArrayList;
import java.util.Random;

import commandcenter.CommandCenter;
import enumerate.Action;
import enumerate.State;
import structs.FrameData;
import structs.GameData;
import structs.Key;
import gameInterface.AIInterface;

public class DeathRose implements AIInterface {
	private Key _key;
	private boolean _ai;
	private FrameData _fd;
	private CommandCenter _command;
	private GameData _gd;

	@Override
	public int initialize(GameData gameData, boolean playerNumber) {
		// TODO Auto-generated method stub
		_key = new Key();
		_ai = playerNumber;
		_fd = new FrameData();
		_command = new CommandCenter();
		_gd = gameData;
		return 0;
	}

	@Override
	public void getInformation(FrameData frameData) {
		// TODO Auto-generated method stub
		_fd = frameData;
		_command.setFrameData(_fd, _ai);
	}

	@Override
	public void processing() {
		// TODO Auto-generated method stub
		if (!_fd.emptyFlag && _fd.getRemainingTime() > 0) {
			double[] temp = GetDistance();
			double dl = temp[0];
			double cos = temp[1];
			State selfState = _command.getMyCharacter().getState();
			State oppState = _command.getEnemyCharacter().getState();
			if (dl < 100) {
				// 直线距离小于100
				if (selfState.equals(State.AIR)||selfState.equals(State.DOWN)) {
					// 如果自己在空中或倒地则防御
					Guard();
				} else {
					if (oppState.equals(State.STAND)
							|| oppState.equals(State.CROUCH)) {
						// 如果对手在地上
						Throw();
					} else {
						// 对手不能抛投,则防御对手出气功波,防御中有攻击技能
						GuardProjectile();
					}
				}
			} else if (dl >= 100 && dl <= 280) {
				if (cos < 0.5 && cos != 1) {
					// 角度大于60度
					if (selfState.equals(State.AIR)||selfState.equals(State.DOWN)) {
						// 如果自己在空中或倒地则防御
						Guard();
					} else {
						// 否则出气功波
						if (oppState.equals(State.AIR)) {
							ProjectileAir();
						} else {
							ProjectileGround();
						}
					}
				} else {
					// 否则出气功波
					if (oppState.equals(State.AIR)) {
						ProjectileAir();
					} else {
						ProjectileGround();
					}
				}
			} else if (dl > 280) {
				// 超过280之后就根据对手状态来出招
				if (oppState.equals(State.AIR)) {
					ProjectileAir();
				} else {
					ProjectileGround();
				}
			}
		}
	}

	@Override
	public Key input() {
		// TODO Auto-generated method stub
		return _key;
	}

	@Override
	public void close() {
		// TODO Auto-generated method stub

	}

	// 释放技能
	private void Cast(Action _action) {
		if (!_command.getskillFlag()) {
			_key.empty();
			_command.skillCancel();
			_command.commandCall(_action.toString());
		} else {
			_key = _command.getSkillKey();
		}
	}

	// 投掷
	private void Throw() {
		Random _random = new Random();
		ArrayList<Action> list = new ArrayList<Action>();
		list.add(Action.THROW_A);
		list.add(Action.THROW_B);
		Cast(list.get(_random.nextInt(list.size())));
	}

	// 对空释放远程技能
	private void ProjectileAir() {
		Cast(Action.STAND_D_DF_FB);
	}

	// 在地面释放远程技能
	private void ProjectileGround() {
		Random _random = new Random();
		ArrayList<Action> list = new ArrayList<Action>();
		list.add(Action.STAND_D_DF_FA);
		if (_command.getMyCharacter().getEnergy() >= 300) {
			list.add(Action.STAND_D_DF_FC);
		}
		Cast(list.get(_random.nextInt(list.size())));
	}

	// 防御远程技能
	private Action GuardProjectile() {
		Random _random = new Random();
		ArrayList<Action> list = new ArrayList<Action>();
		list.add(Action.STAND_D_DB_BA);
		list.add(Action.STAND_D_DB_BB);
		list.add(GetGuard());
		return list.get(_random.nextInt(list.size()));
	}

	// 防御
	private void Guard() {
		Cast(GetGuard());
	}

	// 当前状态下防御
	private Action GetGuard() {
		Action act;
		switch (_command.getMyCharacter().getState()) {
		case STAND:
			act = Action.STAND_GUARD;
			break;
		case DOWN:
		case CROUCH:
			act = Action.CROUCH_GUARD;
			break;
		case AIR:
			act = Action.AIR_GUARD;
			break;
		default:
			act = Action.STAND_GUARD;
			break;
		}
		return act;
	}

	private double[] GetDistance() {
		double dX = _command.getDistanceX();
		double dY = _command.getDistanceY();
		// 直线距离
		double dl = dY <= 0 ? dX : dX <= 0 ? dY : (Math.ceil(Math.sqrt(Math
				.pow(dX, 2) + Math.pow(dY, 2))));
		// 值越大离得越近越高
		double cosVal = dX > 0 && dY > 0 ? (Math.ceil((dX / dl) * 100) / 100)
				: 1;
		return new double[] { dl, cosVal };
	}
}
